import {getDataSet, getEventParam, throttle, toast} from "../../../../utils/utils";
import Api from "../../../../model/api";
import Config from "../../../../model/config";

Component({
    properties: {
        directionData: Array,
        modelData: Array,
        typeData: Array,
    },
    data: {
        directionIndex: -1,
        modelIndex: -1,
        typeIndex: -1,
        time: 1,
        count: 1,
        msg: '',
        attachmentType: '',
        attachmentUrl: '',
        uploadPath: Config.UPLOAD_PATH.COMMUNITY_MANAGEMENT
    },
    methods: {
        bindChange(e) {
            const field = getDataSet(e, "field");
            const value = getEventParam(e, 'value');
            this.setData({
                [field]: value
            })
        },
        onChange(e) {
            const field = getDataSet(e, "field");
            this.setData({
                [field]: getEventParam(e)
            })
        },
        handleUpload(e) {
            const fileType = getEventParam(e, "fileType");
            const fileList = getEventParam(e, "fileList");
            const urls = fileList.map(item => item.url);
            const attachmentUrl = JSON.stringify(urls);
            this.setData({
                attachmentType: fileType, attachmentUrl
            })
        },
        publish: throttle(async function (e) {
            if (this.data.directionIndex == -1) {
                toast('请选择培训方向')
                return;
            }
            if (this.data.modelIndex == -1) {
                toast('请选择培训形式')
                return;
            }
            if (this.data.typeIndex == -1) {
                toast('请选择课程类型')
                return;
            }
            if (!this.data.msg.trim()) {
                toast('请输入内容')
                return;
            }
            const params = {
                trainDirect: this.data.directionData[this.data.directionIndex].dictValue,
                trainForm: this.data.modelData[this.data.modelIndex].dictValue,
                lessonType: this.data.typeData[this.data.typeIndex].dictValue,
                trainTime: this.data.time,
                trainRate: this.data.count,
                imgDesc: this.data.msg,
                attachmentType: this.data.attachmentType,
                attachmentUrl: this.data.attachmentUrl
            }
            await Api.ReleaseNeeds(params, true);
            this.triggerEvent("postFinish");
        })
    }
})
